local CO = require "api_coservice"
local MSG = require "api_msg"
local cjson = require "cjson"

local dset = {}
--------------------------------------
local function API_get(red, args)
	--TODO: Note, no used.
	MSG.OK()
	return true
end

local function API_list(red, args)
	local proxy = CO.GetProxy( "/auth", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		local r,users,err = proxy.GetUsers("web")
		proxy:Destroy()
		if not r or not users then
			-- MSG.ERROR( "Fail to get user! Error:" .. (err or "SYSTEM ERROR") )
			MSG.ERROR( "getUsersError" )
		else
			local rTable = {}
			local k,v
			for k,v in pairs(users) do
				table.insert( rTable, {
					id = k,
					alias = v.name or k,
					web = v.enable_web,
					api = v.enable_api,
					create_time = v.createTime or "1970-01-01 00:00:00"
				})
			end
			table.sort( rTable, function(a,b)
				return a.create_time <= b.create_time
			end)

			MSG.OK("", {
				data = rTable
			})
		end
	end
	return true
end

math.randomseed(string.reverse(tostring(os.time())))
local function get_random_key()
	local rmnumber = os.time() .. math.random(1, 100)
	return ngx.md5(rmnumber .. "XVBKBLL7C9")                                                        
end

local function get_token(user,password,session)
	local token = ngx.md5( "NewTekSanAntonio:" .. user .. ":" .. (password or "") .. session)
	return token
end

local token_dict = {}
local function API_login(red, args, skip)
	if args and args.username and not args.user then
		args.user = args.username
	end

	local proxy = CO.GetProxy( "/auth", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	elseif tostring(args.user) ~= "" and tostring(args.password) ~= "" then
		local r,done,err = proxy.Auth("web", args.user, args.password, skip and "enable_api" or "enable_web")
		if not r or not done then
			-- MSG.ERROR( "Login fail! Error:" .. (err or "SYSTEM ERROR") )
			MSG.ERROR( "loginError" )
		else
			local accept_eula = err and 1 or 0
			r, done, err = proxy.GetUserInfo( "web", args.user )
			local session = get_random_key()
			local token = get_token(args.user,args.password,session)
			local now = os.time()
			MSG.OK( "", {
				data = {
					token = token,
					session = "",
					alias = done and done.name or args.user,
					software = accept_eula, -- 是否同意许可协议
					changed = done.changed -- 判断是否是第一次登陆，是否需要修改密码
				}
			})


			token_dict[token] = {
				time = os.time(),
				user = args.user,
				password = args.password,
				session = session
			}

		end
		proxy:Destroy()
	else
		proxy:Destroy()
		MSG.ERROR( "authCheckError" )
	end
	return true
end

local HTTPAPIAuthorization

--TODO: 定时清除 session_dict中的无效数据
local function check_session(args)

	if HTTPAPIAuthorization ~= nil and HTTPAPIAuthorization == false then
		return true
	end

	local proxy = CO.GetProxy("/", "systemctrl")
	if proxy then
		local _, skip = proxy.GetHTTPAPIAuthorization()
		proxy:Destroy()
		HTTPAPIAuthorization = skip
		if not skip then
			return true
		end
	end

	local token = args["api-token"] or ngx.var["cookie_token"] or (args.head and args.head["api-token"])

	if not token or token == "" then
		ngx.say('{"result":"error","msg":"Invalid token"}')
		return nil
	end

	if not token_dict[token] then
		ngx.say('{"result":"error","msg":"Not exists token infomation"}')
		return nil
	end

	local time = token_dict[token].time
	if not time or (os.time() - time) >= 60 * 10 then
		token_dict[token] = nil
		ngx.say('{"result":"error","msg":"Session is timeout"}')
		return nil
	end
	token_dict[token].time = os.time()


	local user = token_dict[token].user -- 检查用户是否被删除或者密码是否改变
	local proxy = CO.GetProxy("/auth", "systemctrl")
	if proxy then
		local r, done, err = proxy.GetUserInfo( "web", user )
		proxy:Destroy()
		if not r or not done then
			token_dict[token] = nil
			ngx.say('{"result":"error","msg":"Not exists user"}')
		else
			local new_token = get_token(user,done.password,token_dict[token].session)
			if new_token ~= token then
				token_dict[token] = nil
				ngx.say('{"result":"error","msg":"User has been changed"}')
			end
		end
	end

	-- if ngx.md5("NewTekSanAntonio:" .. user .. ":" .. (password or "") .. session) ~= token then
	-- 	ngx.say('{"result":"auth-failed","msg":"Session is not matching with token"}')
	-- 	return nil
	-- end

	

	return true
end

local function API_session(red, args)

	local cookie_token = ngx.var["cookie_token"]
	if not cookie_token or cookie_token == "" then
		MSG.ERROR( "Verfication error: 1" )
	else
		local user = ngx.var["cookie_user"] or (token_dict[cookie_token] and token_dict[cookie_token].user) or ""
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r, done, err = proxy.GetUserInfo( "web", user )
			if not r or not done then
				MSG.ERROR( "Verification error: 2" )
			else
				if get_token(user,done.password,token_dict[cookie_token] and token_dict[cookie_token].session or "") == cookie_token then
					MSG.OK()
				else
					MSG.ERROR( "Verification error: 3" )
				end
			end
			proxy:Destroy()
		end
	end
	return true
end
--------------------------------------

local function API_modify(red, args)
	if args and (args.id or args.username) then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err = proxy.ChangeUser( "web", args.id or args.username, {
				enable_api = (args.api and args.api == "true") ,
				enable_web = (args.web and args.web == "true"),
				password = args.password,
				name = args.alias
			})
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "Fail to modify user, with error: " .. (err or "SYSTEM ERROR") )
				MSG.ERROR( "modifyUserError" )
			end
		end
	else
		MSG.ERROR( "invalidArgError" )
	end
	return true
end

local function API_add(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err = proxy.CreateUser( "web", args.id, {
				enable_api = args.api == "true",
				enable_web = args.web == "true",
				password = args.password,
				name = args.alias
			})
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "Fail to create user with error: " .. (err or "SYSTEM ERROR") )
				MSG.ERROR( "createUserError")
			end
		end
	else
		MSG.ERROR( "invalidArgError" )
	end
	return true
end

local function API_remove(red, args)
	if args and args.ids then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err
			if type(args.ids) == "string" then
				r,done,err = proxy.RemoveUser("web", args.ids )
			elseif type(args.ids) == "table" then
				local i,v
				for i,v in pairs(args.ids) do
					r,done,err = proxy.RemoveUser("web", v )
				end
			else
				r = false
				done = false
				err = "Invalid user ID"
			end

			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "Fail to remove user with error: " .. (err or "SYSTEM ERROR") )
				MSG.ERROR( "delUserError" )
			end
		end
	else
		MSG.ERROR( "invalidArgError" )
	end
	return true
end

local function API_software(red, args)
	if args and (args.id or args.user) then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err = proxy.ChangeUser( "web", args.id or args.user, {
				accept_eula = args.software and tonumber(args.software) ~= 0
			})
			proxy:Destroy()
			MSG.OK("")
		end
	else
		MSG.ERROR( "invalidArgError" )
	end
	return true
end

local function API_software_status(red, args)
	local cookie_user = ngx.var["cookie_user"]


	if not cookie_user and args and args.user then
		cookie_user = args.user
	end

	if not cookie_user or cookie_user == "" then
		MSG.OK( "", {
			data = {
				err_flag = "no-user",
				software = 1 --Don't bother UI.
			}
		})
	else
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.OK( "", {
				data = {
					err_flag = "system",
					user = cookie_user,
					software = 1 --Don't bother UI.
				}
			})
		else
			local r, done, err = proxy.GetUserInfo( "web", cookie_user, true ) --Request EULA status.
			if not r or not done then
				MSG.OK( "", {
					data = {
						err_flag = "verify-user",
						user = cookie_user,
						software = 1 --Don't bother UI.
					}
				})
			else
				MSG.OK( "", {
					data = {
						user = cookie_user,
						software = err and 1 or 0
					}
				})
			end
			proxy:Destroy()
		end
	end
	return true
end

local function API_accept_eula(red, args)
	local user_id
	local token
	if args then
		user_id = args["session-id"] or args["user"]
		token = args["auth-token"] or args["token"]
	end
	if not user_id or not token then
		-- MSG.ERROR( "Verfication error: Invalid arguments" )
		MSG.ERROR("invalidArgError")
	else
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r, done, err = proxy.GetUserInfo( "web", user_id )
			if not r or not done then
				MSG.ERROR( "Verification error: Invalid session/user ID" )
			else
				if get_token(user_id,done.password,"") == token then
					proxy.GlobalAcceptEULA()
					token_dict[token] = {
						time = os.time(),
						user = user_id,
						password = done.password,
						session = ""
					}
					MSG.OK()
				else
					MSG.ERROR( "Verification error: Authenticate failed" )
				end
			end
			proxy:Destroy()
		end
	end
	return true
end

local function API_setVerification(_, args)
	local skip = tostring(args.verification)
	if not skip or skip == "" then
		MSG.ERR("Invalid arguments!")
		return true
	end

	local proxy = CO.GetProxy("/", "systemctrl")
	if not proxy then
		MSG.ERROR("systemError")
		return true
	end

	skip = skip == "true"

	proxy.SetHTTPAPIAuthorization(skip)
	HTTPAPIAuthorization = skip
	MSG.OK("")
	return true
end

local function API_getVerification()
	local proxy = CO.GetProxy("/", "systemctrl")
	if not proxy then
		MSG.ERROR("systemError")
		return true
	end
	local is_ok, skip = proxy.GetHTTPAPIAuthorization()
	if not is_ok then
		MSG.ERROR("Fail to get Authorization infomation ")
		return true
	end
	HTTPAPIAuthorization = skip
	MSG.OK("", {data = {verification = skip}})
	return true
end

--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["modify"] = API_modify
dset.APIS["login"] = API_login
dset.APIS["session"] = API_session
dset.APIS["list"] = API_list
dset.APIS["add"] = API_add
dset.APIS["remove"] = API_remove
dset.APIS["software"] = API_software
dset.APIS["software_status"] = API_software_status
dset.APIS["accept_eula"] = API_accept_eula
dset.APIS["setVerification"] = API_setVerification
dset.APIS["getVerification"] = API_getVerification
dset.APIS["authorize"] = function(_, args)
	return API_login(_, args, true)
end
dset.check_session = check_session
--------------------------------------
return dset




