local _dbg = require "coaf.debug"
local cjson = require "cjson"
local pretty = require "pl.pretty"
local copy_merge = require "PtzManager.copy_merge"
local lfs = require "lfs"
local socket = require "socket"
local ev = require "ev"
require "ndi_encoding"
----------------------------------------------
local _M = {
	ptz_obj = nil,
	ptz_protocol_list = {["Sony Visca"]="visca",["Visca (for Kato)"]="visca_kato",["Visca (for Laia)"]="visca_laia",["Pelco-d"]="pelco_d",["Pelco-p"]="pelco_p"}

}

----------------------------------------------
local SOCKET_SVC = {
	version = "1.0",
	svc_socket = nil,
	loop = nil,

	clients = {
	}
}
----------------------------------------------

local function StartSerialRxMonitor()
	if not _M.ptz_obj then
		_dbg.Error( "SERIAL_SERVICE: Invalid serial/ptz object while starting Rx watcher." )
		return
	end

	if _M.serial_rx_watcher then
		_dbg.Error( "SERIAL_SERVICE: The serial Rx watcher is already started." )
		return
	end

	_M.serial_rx_watcher = ev.IO.new( function( loop, watcher )
		local ser = _M.ptz_obj and _M.ptz_obj:GetSerialObject() or nil
		if not ser then
			_dbg.Error( "SERIAL_SERVICE: Invalid serial/ptz object, so closed the watcher!" )
			watcher:stop(loop)
			_M.serial_rx_watcher = nil
			return
		end

		local data
		local status, err = pcall( function() data = ser:read(128,0) end )

		if true then
			return
		end
		if not status then
			_dbg.Error( "SERIAL_SERVICE: Read serial port error! Stopped the watcher!" )
			watcher:stop(loop)
			_M.serial_rx_watcher = nil
			return
		end
	end, _M.ptz_obj:GetSerialObject().fd, ev.READ )

	if not _M.serial_rx_watcher then
		_dbg.Error( "SERIAL_SERVICE: Warning! Fail to start serial Rx watcher!" )
		return
	end

	_M.serial_rx_watcher:start( SOCKET_SVC.loop )
end


local function StopSerialRxMonitor()
	if _M.serial_rx_watcher then
		if _M.serial_rx_watcher:is_pending() then
			_M.serial_rx_watcher:clear_pending()
		end
		_M.serial_rx_watcher:stop( SOCKET_SVC.loop )
		_M.serial_rx_watcher = nil
	end
end


local function network_ptz(tabs)
	PTZ = require "net_ptz"

	_M.ptz_obj = PTZ.CreateNew( tabs.network.protocol, tabs.network.addr, tabs.network.port )

	if _M.ptz_obj then
		if _M.ptz_protocol_list[tabs.network.ptz_protocol] then
			_M.ptz_obj:LoadProtocol(_M.ptz_protocol_list[tabs.network.ptz_protocol])
		else
			_M.ptz_obj:LoadProtocol("visca")
		end
		_M.ptz_obj:SetAddress(tonumber(tabs.network.ptz_addr))
	end
	return true 
end                       


local function serial_ptz(tabs)
	local PTZ = require "ptz"
	_M.ptz_obj = PTZ.CreateNew({	device = tabs.serial.device, 
						baudrate = tabs.serial.baudrate, 
						databits = tabs.serial.startBits, 
						parity = tabs.serial.parity, 
						stopbits = tabs.serial.endBits, 
						xonxoff = (tabs.serial.xonxoff == 1),
						rtscts = (tabs.serial.rtscts == 1)
					 })
	if _M.ptz_obj then
		-- StartSerialRxMonitor()
		if _M.ptz_protocol_list[tabs.serial.ptz_protocol] then
			_M.ptz_obj:LoadProtocol(_M.ptz_protocol_list[tabs.serial.ptz_protocol])
		else
			_M.ptz_obj:LoadProtocol("visca")
		end
		_M.ptz_obj:SetAddress(tonumber(tabs.serial.ptz_addr))
	end
	return true 

end 

function _M.Choise()

	if _M.ptz_obj then
		-- StopSerialRxMonitor()
		_M.ptz_obj:Close()
		_M.ptz_obj = nil
	end
	local r, err	
	if cfg_ptz.typ=="network" then
		r, err = network_ptz(cfg_ptz)
	elseif cfg_ptz.typ=="serial" then
		r, err = serial_ptz(cfg_ptz)
	else
		return false, "error ptz type"
	end

	if not r then
		return false, err
	end
	return true
end


function _M.Modify(details)	
	copy_merge( cfg_ptz, details )
	cfg_ptz:__TRIGGER(true)

	if cfg_ptz.enable == 0 then
		if _M.ptz_obj then
			-- StopSerialRxMonitor()
			_M.ptz_obj:Close()
			_M.ptz_obj = nil
		end
		NDI.set_enable_ptz(false)
		return true
	else
		NDI.set_enable_ptz(true)
	end

	local r,err = _M.Choise()
	if not r then
		return false,err
	end
	return true
end


function _M.GetPtzInfo()
	ptz_cfg = require("PtzManager.configs.ptz_configs").Load()
	return ptz_cfg
end


function _M.Getdevices()
	local USBSerials = require "PtzManager.sysStatus"
	return USBSerials.GetUSBSerials()

end


function _M.GetPtzProtocol()

	local data = {"Sony Visca","Visca (for Kato)","Visca (for Laia)","Pelco-d","Pelco-p"}
	return data

end


function _M.Control(action,params)
	if action == "set-address" then
		if _M.ptz_obj then
			local addr = 1
			if params then
				if params["address"] then
					addr = tonumber(params["address"])
				elseif params["addr"] then
					addr = tonumber(params["addr"])
				end
			end
			_M.ptz_obj:SetAddress(addr)
			return 0,{}
		else
			return -1,{}
		end
	elseif action == "load-protocol" then
		if _M.ptz_obj then
			_M.ptz_obj:LoadProtocol( params and params["protocol"] or "pelco_d" )
			return 0,{}
		else
			return -1,{}
		end
	elseif action == "set-speed" then
		if _M.ptz_obj then
			_M.ptz_obj:SetSpeed( params )
			return 0,{}
		else
			return -1, {}
		end
	elseif action == "save-preset" then
		if _M.ptz_obj then
			--TODO: Mark presets
			_M.ptz_obj:SavePreset( params and params["id"] or 0 )
			return 0,{}
		else
			return -1, {}
		end
	elseif action == "remove-preset" then
		if _M.ptz_obj then
			--TODO: Mark presets
			_M.ptz_obj:RemovePreset( params and params["id"] or 0 )
			return 0,{}
		else
			return -1, {}
		end
	elseif action == "load-preset" then
		if _M.ptz_obj then
			--TODO: Mark presets
			_M.ptz_obj:LoadPreset( params and params["id"] or 0 )
			return 0,{}
		else
			return -1, {}
		end
	else
		if _M.ptz_obj then
			local arg1 = nil
			local arg2 = nil
			if params then
				if params["speed"] then
					arg1 = tonumber(params["speed"])
				elseif params["pan_speed"] then
					arg1 = tonumber(params["pan_speed"])
				elseif params["pan"] then
					arg1 = tonumber(params["pan"])
				elseif params["id"] then
					arg1 = tonumber(params["id"])
				end

				if params["tilt_speed"] then
					arg2 = tonumber(params["tilt_speed"])
				elseif params["tilt"] then
					arg2 = tonumber(params["tilt"])
				elseif params["speed2"] then
					arg2 = tonumber(params["speed2"])
				end
			end
			local r = _M.ptz_obj:DoAction( action, _M.ptz_obj:GetActionSpeedArgs(action,arg1,arg2) )
			if r then
				return 0, {}
			else
				return -1, {}
			end
		else
			return -1, {}
		end
	end
end


function _M.Check()
	local attr = lfs.attributes(cfg_ptz.serial.device)
	if cfg_ptz.enable == 0 then
		return
	end
	if cfg_ptz.typ == 'network' then
		return
	end
	if not attr then --Device is not exist.
		if _M.ptz_obj then
			-- StopSerialRxMonitor()
			_M.ptz_obj:Close()
			_M.ptz_obj = nil
		end
		return
	end

	if attr and not _M.ptz_obj then
		_M.Choise()
	end
end




function _M.Init(loop)
	SOCKET_SVC.loop = loop
	_M.Choise()
end


return _M

--[[
--PTZ = require "net_ptz"
--p = PTZ.CreateNew( "tcp", "192.168.2.48", 5678 )
--p:LoadProtocol("visca")
--p:SetAddress(1)
--
----p:MoveLeft()
----p:StopMove()
----p:MoveRight()
----p:StopMove()
--
--
--root@832de0bd5e0b:/solar/COMMON/SUN/ptzModule# cat test/test_ptz.lua 
--PTZ = require "ptz"
--p = PTZ.CreateNew( "/dev/ttyUSB0", 9600 )
--p:LoadProtocol("visca")
--p:SetAddress(1)
--
--]]
