local coaf = require "coaf.core"
local thread = require "coaf.thread"
local _dbg = require "coaf.debug"
--local I2C_REG = require "chips.i2c_reg"
local periphery = require "periphery"
local GPIO = periphery.GPIO

local _M = {
}

-------------------------------------------------------------
-- The FAN
-------------------------------------------------------------
local fan_meta = {
}

function fan_meta:read()
	return self.fan_speed or 0
end

function fan_meta:write(val)
	if type(val) == "boolean" or val == nil then
		self:enable(val and 100 or 0)
	else
		self:enable(val)
	end
end

function fan_meta:enable(speed)
	if not self.pin_drv then
		self.pin_drv = GPIO( self.pin, "out" )
	end
	--TODO: Speed control.
	speed = tonumber(speed) or 0
	self.fan_speed = speed
	if self.pin_drv then
		if speed > 0 then
			self.pin_drv:write( self.enable_level ) --Enable.
		else
			self.pin_drv:write( not self.enable_level ) --Disable
		end
	end
end

local function fan_control_task(thr_obj, fan_obj)
	local watch_index, speed
	local overthreshold
	local flag = 0
	while not thr_obj.terminated do
		local temp = fan_obj.temp_chip:read()
		if flag < 15 then
			flag = flag +1
			fan_obj:enable(100)
			if flag >= 15 then
				fan_obj:enable(0)
			end
		elseif not temp then
			fan_obj:enable(100) --Avoid too hot!
		else
			watch_index = nil
			overthreshold = false
			for i, item in ipairs(fan_obj.speed_table) do
				if temp < item.temp then
					watch_index = i
					speed = item.speed
					break
				end
				if item.threshold then
					overthreshold = true
				end
			end

			if not watch_index then --Too hot! 100% run!
				if fan_obj.fan_speed <= 0 then
					_dbg.Info("FAN", "Temperature =", temp, " is too hot! start the fan with speed 100%")
				end
				fan_obj:enable(100)
			elseif watch_index == 1 then -- Cold, disable run.
				if fan_obj.fan_speed > 0 then
					_dbg.Info("FAN", "Temperature =", temp, " is cold, stop the fan.")
				end
				fan_obj:enable(0) 
			else
				if overthreshold then
				--It's hot over the threshold, start FAN.
					if fan_obj.fan_speed <= 0 then
						_dbg.Info("FAN", "Temperature =", temp, ", start the fan with speed", speed, "%")
					end
					fan_obj:enable(speed)

				elseif fan_obj.fan_speed > 0 then
				--Enable FAN only if it previously started.
				--That is, for example:
				--                   threshold
				--                       |
				--  40  41  42  43  44  <45>  46  47 ...
				--
				-- If previous is not started, temp=42, don't start fan (it's hot-safe yet)
				-- But if previous is already started, temp=42, since it's over threshold before,
				-- let it continue run until less than 40 deg.
					fan_obj:enable(speed)
				end
			end
		end
		coaf.Sleep(2)
	end
end

--Speed table:
--[[
{
	--Lower than, disable.
	{temp = 40, speed = 10},
	{temp = 42, speed = 20},
	{temp = 45, speed = 30, threshold = true},
	{temp = 48, speed = 50},
	{temp = 50, speed = 60},
	{temp = 52, speed = 70},
	{temp = 55, speed = 80},
	{temp = 58, speed = 100},
	--Upper than, 100%
}
--]]

function fan_meta:start(speed_table)
	if type(speed_table) ~= "table" then
		speed_table = {
			{temp = 40, speed = 10},
			{temp = 42, speed = 20},
			{temp = 45, speed = 30, threshold = true},
			{temp = 48, speed = 50},
			{temp = 50, speed = 60},
			{temp = 52, speed = 70},
			{temp = 55, speed = 80},
			{temp = 58, speed = 100}
		}
	end

	self.speed_table = speed_table
	if self.temp_chip then
		--Disable fan first.
		self:enable(0)
		if not self.fan_task then
			self.fan_task = thread.New(fan_control_task)
			self.fan_task:Start(self)
		end
	else
		self:enable(0) --No temperature chip, so disable it first.
		-- you should call :enable(speed) to control the fan.
	end
end

fan_meta.__index = fan_meta
-------------------------------------------------------------

--Create a GPIO based fan object:
function _M.GPIOFan(temp_chip, pin, enable_level)
	if enable_level == nil then
		enable_level = true
	end
	local fan_obj = {
		temp_chip = temp_chip,
		pin = pin,
		enable_level = enable_level,
		fan_speed = 0
	}

	setmetatable(fan_obj, fan_meta)
	return fan_obj
end

return _M
