local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local I2C_REG = require "chips.i2c_reg"

local _M = {
	--Store all I2C transfer objects, index based on I2C device.
	i2c_trans = {}
}

-------------------------------------------------------------
-- The CHIP
-------------------------------------------------------------
local chip_meta = {
}

function chip_meta:read()
	local val, msg = self.i2c_trans:ReadReg(self.address, 0x00, {0,0})
	if type(val) == "table" then
		local temp = val[1]
		temp = (((temp << 8) + val[2]) >> 5) & 0x7ff
		if (temp & 0x400) == 0 then
			return temp*0.125
		else
			temp = (~temp + 1)&0x3ff
			return -temp*0.125
		end
	else
		return nil
	end
end

chip_meta.__index = chip_meta
-------------------------------------------------------------

--Create a chip:
function _M.Chip(i2c_dev, i2c_addr)
	i2c_addr = tonumber(i2c_addr)
	if not i2c_dev or not i2c_addr then
		_dbg.Error("LM75ADP", "Invalid I2C device or device address!")
		return nil
	end

	local i2c_trans = _M.i2c_trans[i2c_dev]
	if not i2c_trans then
		i2c_trans = I2C_REG.Create()
		if not i2c_trans:Open(i2c_dev) then
			_dbg.Error("LM75ADP", "Open I2C device failed! device=", i2c_dev, ", address=", string.format("%02X",i2c_addr))
			return nil
		end
		_dbg.Info("LM75ADP", "Opened temperature controller chip at device=", i2c_dev, ", address=", string.format("%02X",i2c_addr))
		_M.i2c_trans[i2c_dev] = i2c_trans
	else
		_dbg.Info("LM75ADP", "Using exist I2C transfer object for ", i2c_dev)
	end

	local chip_obj = {
		i2c_trans = i2c_trans,
		address = i2c_addr
	}

	setmetatable(chip_obj, chip_meta)
	return chip_obj
end

return _M
