local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"
local client = require "coaf.client"

--[[
    此模块用于兼容 mutiview录制系统的 ntp授时功能，用于像metadata发送数据
]]


local _M = {
	cfg = nil,
    date_proxy = nil,
    server = nil
}

local function split(str, sep)
    local sep, fields = sep or "\t", {}
    local pattern = string.format("([^%s]+)", sep)
    str:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

local function parseStatus(list)
    local new = {}
    local tbl = split(list, "\n")
    if tbl[1] and tbl[2] then
        table.remove(tbl, 2)
		table.remove(tbl, 1)
    end

    -- pt.dump(tbl)

    for _, v in ipairs(tbl) do
		local tmp = split(v, " ")
		-- pt.dump(tmp)
		if tmp then
			if #tmp == 9 then
				table.insert(new, {                                                    
						remote = tmp[1],
				refid = "",
				st = tmp[2],
				t = tmp[3],
				when = tmp[4], 
				poll = tmp[5], 
				reach = tmp[6],   
				delay = tmp[7],  
				offset = tmp[8],  
				jitter = tmp[9],
					})
			elseif #tmp == 10 then
						table.insert(new, {
						remote = tmp[1],   
				refid = tmp[2],
						st = tmp[3],       
						t = tmp[4],                                            
						when = tmp[5],                                       
						poll = tmp[6],                                    
						reach = tmp[7],                              
						delay = tmp[8],                                             
						offset = tmp[9],                                                                          
						jitter = tmp[10],                                                                          
						})
			end
		end
    end

    return new
end




local function get_sysdate_proxy()
	if _M.date_proxy then
		return _M.date_proxy
	end
	local proxy
	local cli
	cli = client.New( "systemctrl", "local", nil, 5 ) --5 seconds timeout.
	if not cli then
		_dbg.Error( "SYS-datetime", "Fail to create connection to service 'local/systemctrl'" )
		return nil
	end
	proxy = cli:CreateProxy("/datetime", true) --Exclusive
	if not proxy then
		_dbg.Error( "SYS-datetime", "Fail to create object proxy for '/auth@local/systemctrl'" )
		cli:Destroy()
		return nil
	end
	_M.date_proxy = proxy
	return proxy
end




function _M.GetNTPStatus()
    if not _M.server then
        _M.server = ""
    end
    local cmd = "/usr/bin/ntpq -pn 2>/dev/null" -- 丢弃标准错误输出(当ntp没有启动的时候，会报错)
    local f = io.popen(cmd)
    local list = parseStatus(f:read("*a"))
    f:close()

    local tal = {sync=0,remote=""}
    for k,v in pairs(list) do
        if v and v.remote then
            tal.remote = _M.server
            if string.find(v.remote,"*") and v.offset and v.jitter and v.poll and v.when and not string.find(v.refid,"LOCL") then
				
                if v.when == "-" then
                    v.when = 0
                end
                if math.abs(tonumber(v.offset))<100 and math.abs(tonumber(v.jitter))<100 and tonumber(v.when)<(tonumber(v.poll)+8) then
                    tal.sync = 1
                end
            end
        end
    end
    return tal
end

-- 设置ntp服务器，并开启ntp服务
function _M.SetNtpServer(server)
    local s = server
    if type(server)=="table" then
        s = ""
        for k,v in pairs(server) do
            s = s .. v .. " "
        end
    end
    local proxy = get_sysdate_proxy()
    if not proxy then
        _dbg.Error( "CODEC", "Fail to get systemctrl datetime proxy!" )
    else
        proxy.SetNtpServer(server)
    end
    _M.server = s
end

function _M.Init()
end

return _M

