#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local new_vercode = nil
local firmware_exist = false
local periphery = require "periphery"
local I2C = periphery.I2C
local socket = require("socket")

local I2C_DEV = "/dev/i2c-1"
local BASE_ADDR = 0xD2
local SAVED_VERSION = nil


function ReadReg( addr, itemArr )
    local i2c_handle
	local done,err = pcall( function() i2c_handle = I2C( I2C_DEV ) end )
	if not done then
		if type(err) == "table" then
			_dbg.Error( "HW", "Could not open I2C device for FPGA controlling. ErrMsg:", err.message or "---" )
		else
			_dbg.Error( "HW", "Could not open I2C device for FPGA controlling. ErrMsg:", err )
		end
		return nil
	else
		_dbg.Info( "HW", "FPGA controller openned, at " .. string.format("%02x",BASE_ADDR) .. "@" .. I2C_DEV )
	end

	if not itemArr or type(itemArr) ~= "table" then
		itemArr = { 0xff, flags = I2C.I2C_M_RD }
	else
		if #itemArr < 1 then
			itemArr[1] = 0xff
		end
		if not itemArr.flags then
			itemArr.flags = I2C.I2C_M_RD
		else
			itemArr.flags = itemArr.flags | I2C.I2C_M_RD
		end
	end

	local addrs = {}
	local i
	for i=1, #itemArr do
		addrs[i] = addr+i-1
	end

	local msg = { addrs, itemArr }
	local done,err = pcall( function() i2c_handle:transfer( BASE_ADDR>>1, msg ) end )
	i2c_handle:close()
	if not done then
		if type(err) == "table" then
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err )
			return nil, err
		end
	else
		return itemArr
	end
end


local gpio983 = periphery.GPIO(983,"out") -- 初始化设置gpio76拉低
if gpio983 then
	gpio983:write(false)
end



local file = io.open( "/usr/local/bin/fpga_upgrade/fpga_firmware/VERSION", "r" )
if file then
	local line = file:read( "*l" )
	file:close()
	if line then
		new_vercode = tonumber( "0x" .. line )
	end
end

file = io.open( "/usr/local/bin/fpga_upgrade/fpga_firmware/firmware.bin", "rb" )
if file then
	firmware_exist = true
	file:close()
end

socket.select(nil,nil,5) -- 等待5秒启动
if new_vercode and firmware_exist then
	local ret,err = ReadReg( 0xa0, {0x00} )
	if not ret and not err then -- Could not open I2C
		os.exit(0)
	end
	if not ret or  ret[1] < new_vercode then
		os.execute( "/usr/local/bin/fpga_upgrade/progfpga.lua /usr/local/bin/fpga_upgrade/fpga_firmware/firmware.bin" )
	else
		print( "*** Upgrading FPGA: The FPGA version is newest." )
	end
end

os.exit(0)

