#!/usr/bin/env lua

local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local service = require "coaf.service"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local thread = require "coaf.thread"

local lfs = require "lfs"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/img_manager.log")
	if not logF or not logF:init() then
		_dbg.Error("CODEC-MATE", "Fail to open log file: /var/log/img_manager.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "img_manager" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "img_manager" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

-- ** CoService **
local cosvc_CodecMate
local sure_codecServiceWorking = false
local max_trytimes = 5
local min_trytimes = 2
local try_times = max_trytimes

local media_max_trytimes = 5
local media_min_trytimes = 3
local media_try_times = media_min_trytimes
local image_config = require "image_config"
local PICPATH = "/data/upload_image/"
local PICPATH_DEFAULT = "/data/upload_image_default/"
local CONV_NDI_IMAGE = "/usr/local/bin/conv_ndi_image"

_G.img_cfg = image_config.Load()
------------------
-- ** A simple service **
------------------
local img_service = {}


function img_service.AddPic(details)

	if _G.img_cfg and details.name and _G.img_cfg[details.name] then

		if details.local_cmd and details.url then
			os.execute("/bin/cp -f \"" .. details.local_cmd .. "\"  " .. PICPATH.. details.url)
		end
		_G.img_cfg[details.name].url = details.url
		_G.img_cfg[details.name].size_w = details.size_w
		_G.img_cfg[details.name].size_h = details.size_h
		_G.img_cfg:__TRIGGER(true)

		os.execute(CONV_NDI_IMAGE .. " --in "..PICPATH .. details.url .. " --out ".. PICPATH .. details.name .. ".ndi --out-preview ".. PICPATH .. details.name .. "_preview.ndi")

		return true
	else
		return false
	end

    return true
end

function img_service.GetPics()

	local data = {}
	for k,v in pairs(_G.img_cfg) do
		local z = v
		z["name"] = k
		table.insert(data,z)
	end
	return data
end

function img_service.ResetPic(name)
	if name then
		if _G.img_cfg and _G.img_cfg[name] then
			os.execute("/bin/rm "..PICPATH .. _G.img_cfg[name].url)
			os.execute("/bin/rm "..PICPATH .. name .. ".ndi")
			os.execute("/bin/rm "..PICPATH .. name .. "_preview.ndi")
			os.execute("/bin/cp " .. PICPATH_DEFAULT .._G.img_cfg[name].url ..  " " .. PICPATH)
			_G.img_cfg[name].size_h = 1080
			_G.img_cfg[name].size_w = 1920
		end
	else
		os.execute("/bin/rm "..PICPATH .. "*")
		os.execute("/bin/cp " .. PICPATH_DEFAULT .. "* " .. PICPATH)

		for k,v in pairs(_G.img_cfg) do
			v.size_h = 1080
			v.size_w = 1920
		end
	end
	_G.img_cfg:__TRIGGER(true)
	return true
end

function img_service.Init()
	local res = lfs.attributes(PICPATH)
	if not res then
		os.execute("/bin/cp -rf " .. PICPATH_DEFAULT .. " " .. PICPATH)
	end
	return true
end


-- **Main entry**
local function Main(args)
	img_service.Init()
	cosvc_CodecMate = service.New( "img_manager", "unix:/var/run/codec_mate.sock" )
	if cosvc_CodecMate then
		_dbg.Info( "img_manager", "CoService of 'img_manager' is created." )
		cosvc_CodecMate:CreateCoServerProxy()
		cosvc_CodecMate:ExposeObject("/", img_service )
	else
		_dbg.Error( "img_manager", "Fail created CoService!" )
	end

end

coaf.Start(Main, arg)


_dbg.Print( "img_manager", "Service terminated." )

