#!/bin/sh

export PROD_ID="N3-s"
export TARGET_TMP_DIR="/upgrade/.upgrade_tmp"

export SWVER="2.02.0227"
export LUA_VERSION="5.3"

export ROOT_REQ_KB="16192"
export DATA_REQ_KB="10720"
export UPGRADE_RETURNS=1

#!/bin/sh
export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
mkdir -p /data/update_log
mkdir -p $TARGET_TMP_DIR/_UPGRADE

updatetime=`date "+%Y-%m-%d %H:%M:%S"`
updatetimecode=`date "+%Y%m%d%H%M"`
WITH_WARNING=0
WITH_ERROR=0
logfile="/data/update_log/$SWVER-$updatetimecode.log"

echo "Upgrade to Software version $SWVER..." > $logfile
echo "Upgrade time: $updatetime" >> $logfile

#NOTE: Must disable watch-dog while upgrading!
if [ -x /sbin/wdog ]; then
	/sbin/wdog disable
	echo "Disabled watch-dog while upgrading." >> $logfile
fi

echo "------------------------------------" >> $logfile

if [ $# -gt 0 ]; then
    cd $1
else
    cd /upgrade
fi

tar xzf ./patch.tar.gz -C $TARGET_TMP_DIR/_UPGRADE 
if [ $? -ne 0 ]; then
	echo "Upgrading fail! Error to extract the patch file!" >> $logfile
	rm -Rf $TARGET_TMP_DIR
	exit 1
fi

rm -Rf ./patch.tar.gz
cd $TARGET_TMP_DIR	#Change to target position

getDiskFree()
{
	#Because the K2E busybox system only support human format
	local freeSpaces=`df -h $1 | sed 1d | awk '{print $4}'`
	if [ "${freeSpaces/G/}" != "${freeSpaces}" ]; then
		#Is GB
		freeSpaces=${freeSpaces/G/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024 * 1024))"`
	elif [ "${freeSpaces/M/}" != "${freeSpaces}" ]; then
		#Is MB
		freeSpaces=${freeSpaces/M/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024))"`
	elif [ "${freeSpaces/K/}" != "${freeSpaces}" ]; then
		#Is KB
		freeSpaces=${freeSpaces/K/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces))"`
	fi
	echo $freeSpaces
	exit 0
}

root_free_sz=`getDiskFree /`
data_free_sz=`getDiskFree /data`

root_free_sz=`expr $root_free_sz - 1024`	#Keep 1MB at least
data_free_sz=`expr $data_free_sz - 1024`	#Keep 1MB at least

echo "ROOT: Require $ROOT_REQ_KB KB, valid $root_free_sz KB." >> $logfile
echo "DATA: Require $DATA_REQ_KB KB, valid $data_free_sz KB." >> $logfile

if [ $root_free_sz -lt $ROOT_REQ_KB -o $data_free_sz -lt $DATA_REQ_KB ]; then
	echo "ERROR: Insufficent disk spaces" >> $logfile
	exit 1
fi

upgrade()
{
	if [ -d "$TARGET_TMP_DIR/_UPGRADE/fakeroot" ]; then
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/fakeroot/"* / >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/datafs" ]; then
		if [ ! -d /data ]; then
			mkdir /data
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/datafs/"* /data/ >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/boot" ]; then
		this_part_id="OTHER"
		default_boot_for_part_id="OTHER"

		if [ -f /boot/PART_ID ]; then
			this_part_id=`cat /boot/PART_ID`
		fi
		if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" ]; then
			default_boot_for_part_id=`cat $TARGET_TMP_DIR/_UPGRADE/boot/boot.def`
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" 
		fi

		#Don't touch PART_ID, if exists.
		#rm "$TARGET_TMP_DIR/_UPGRADE/boot/PART_ID"

		#2 cases: 1) part_id matched; 2) both part_id and 'boot.def' are empty ("OTHER" identified).
		if [ "$default_boot_for_part_id" = "$this_part_id" ]; then
			#Keep default BOOT.BIN, drop others.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
		elif [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" ]; then
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			mv "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			fi

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
			fi
		else
			#Don't touch BOOT.BIN and other .bit files.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit*.bin
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/boot/"* /boot/ >> $logfile 2>&1
	fi

	return 0
}

drop()
{
	if [ "$1" = "fakeroot" ]; then
		if [ -e "$2" ]; then
			rm -Rf "$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	elif [ "$1" = "datafs" ]; then
		if [ -e "/data$2" ]; then
			rm -Rf "/data$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [/data$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	fi
    return 0
}

## The upgrading content below: (generated by release tool)

drop datafs "/web/html/static/css/app.28340a58624decafaef92abf2e7ae7b9.css"
drop datafs "/web/html/static/css/app.403344fb9931fe337946b6adf853d536.css"
drop datafs "/web/html/static/css/app.4b4bf550555502b0e3c2c319dbb25467.css"
drop datafs "/web/html/static/css/app.55d73c5a12f0e229b0d6416cc0a02a15.css"
drop datafs "/web/html/static/css/app.5a62c29952d1852f39924855dece0e88.css"
drop datafs "/web/html/static/css/app.6b4126cfc19cf6e3a17f4dae46c4c8dd.css"
drop datafs "/web/html/static/css/app.6e017a63b3ee60d79214746f7bac58d3.css"
drop datafs "/web/html/static/css/app.771e7751fb453348a429d5e1fccd6282.css"
drop datafs "/web/html/static/css/app.77f302dd6e88e4e3a144146a40d64409.css"
drop datafs "/web/html/static/css/app.78ff4aca6b89ca23eab3cdce4bfe3aa8.css"
drop datafs "/web/html/static/css/app.7bc624137bb49d5c8e7347bc64028856.css"
drop datafs "/web/html/static/css/app.89000dbeec31853424c4c73d1938342e.css"
drop datafs "/web/html/static/css/app.8e936d764db87dc81e3dbe05d0b4d000.css"
drop datafs "/web/html/static/css/app.915d706432b06b6224dc9d8cf23b189f.css"
drop datafs "/web/html/static/css/app.a289dabdc0b4189016ef9a0d3c641296.css"
drop datafs "/web/html/static/css/app.a428b37267c28b0faa0e9e81aa2dcb6f.css"
drop datafs "/web/html/static/css/app.ab4fe621574a922413a9579cd38f1d46.css"
drop datafs "/web/html/static/css/app.b040639a3ade7abcdf6d58ea2ecd1d52.css"
drop datafs "/web/html/static/css/app.b237f502a3a3f06562b7e69ba62a9a9f.css"
drop datafs "/web/html/static/css/app.c5067c67db82a5e777a9a5b708d36eca.css"
drop datafs "/web/html/static/css/app.cd22041bbeb2b113ea19f72d624223cb.css"
drop datafs "/web/html/static/css/app.f7e910b1d432e2aceaace2b26167ccd2.css"
drop datafs "/web/html/static/img/1.381ae97.png"
drop datafs "/web/html/static/img/3.1b7dc9b.png"
drop datafs "/web/html/static/img/4.b6b8a88.png"
drop datafs "/web/html/static/img/5.f0d630c.png"
drop datafs "/web/html/static/img/6.a619365.png"
drop datafs "/web/html/static/img/6.edd7c89.png"
drop datafs "/web/html/static/img/logo.c37e32b.png"
drop datafs "/web/html/static/img/logo.*.png"
drop datafs "/web/html/static/js/10.f6da5cc9e2cfe6f78e97.js"
drop datafs "/web/html/static/js/app.038c38b3987c6aab8a73.js"
drop datafs "/web/html/static/js/app.12058e54bb2bb57e0e42.js"
drop datafs "/web/html/static/js/app.1b6a96afa36a66816fcd.js"
drop datafs "/web/html/static/js/app.397fb53ac0b29122bebd.js"
drop datafs "/web/html/static/js/app.4115b7904b09cacc9e49.js"
drop datafs "/web/html/static/js/app.413c4341d5fd2e9eebde.js"
drop datafs "/web/html/static/js/app.47b2a5d5d7470532b5fe.js"
drop datafs "/web/html/static/js/app.4c311aa71e64bcd3e1e0.js"
drop datafs "/web/html/static/js/app.5643f260484249776625.js"
drop datafs "/web/html/static/js/app.58823bda59f9a2d4be39.js"
drop datafs "/web/html/static/js/app.5bc2ace5a80ae76479c5.js"
drop datafs "/web/html/static/js/app.6d5c9a1f5d523daa3c00.js"
drop datafs "/web/html/static/js/app.96dd853df2efe6bd57a3.js"
drop datafs "/web/html/static/js/app.a086bafee1d69ad73b40.js"
drop datafs "/web/html/static/js/app.ab103b6f47c47cdfc333.js"
drop datafs "/web/html/static/js/app.c76e02c14883e2547a19.js"
drop datafs "/web/html/static/js/app.fc3d821adfa0ec48ab49.js"
drop datafs "/web/html/static/js/decoderMode.01e1359ccd118c1fed3d.js"
drop datafs "/web/html/static/js/decoderMode.044217fc4280a976491c.js"
drop datafs "/web/html/static/js/decoderMode.0a5c7af6300c5ad76f2c.js"
drop datafs "/web/html/static/js/decoderMode.0af4b38727e6c938ffe1.js"
drop datafs "/web/html/static/js/decoderMode.1c3db652b8802e3abe40.js"
drop datafs "/web/html/static/js/decoderMode.27527feda4e833c74f27.js"
drop datafs "/web/html/static/js/decoderMode.6231cedeec00a880b2ad.js"
drop datafs "/web/html/static/js/decoderMode.7495546bbd3a9ce4f5de.js"
drop datafs "/web/html/static/js/decoderMode.be92c68b7c7057713eed.js"
drop datafs "/web/html/static/js/decoderMode.d191bb2d18a941405a34.js"
drop datafs "/web/html/static/js/decoderMode.d4c8fe9431cd4add05d9.js"
drop datafs "/web/html/static/js/decoderMode.e611d97a09076de7242f.js"
drop datafs "/web/html/static/js/decoderMode.fddc513be8ad61541aba.js"
drop datafs "/web/html/static/js/encoderMode.0c59a98133cc16f59e85.js"
drop datafs "/web/html/static/js/encoderMode.1528099c170e882d2823.js"
drop datafs "/web/html/static/js/encoderMode.3e20c7a53e7fcc3ffde2.js"
drop datafs "/web/html/static/js/encoderMode.417854db966b386f4138.js"
drop datafs "/web/html/static/js/encoderMode.42614772293ce6043a03.js"
drop datafs "/web/html/static/js/encoderMode.4bcedb033e3585a1c8de.js"
drop datafs "/web/html/static/js/encoderMode.59c5a02067f5966aa6d2.js"
drop datafs "/web/html/static/js/encoderMode.6ab8a0746c5f0956fea8.js"
drop datafs "/web/html/static/js/encoderMode.940667e911ae628db9ea.js"
drop datafs "/web/html/static/js/encoderMode.b2025aac19934eb88221.js"
drop datafs "/web/html/static/js/encoderMode.d3d9a97b94871ed22222.js"
drop datafs "/web/html/static/js/encoderMode.faf1c61c24b55355bc46.js"
drop datafs "/web/html/static/js/firmwareudp.3e240eda5a272d89c3e0.js"
drop datafs "/web/html/static/js/firmwareudp.4cec59d80369e9fbd0d0.js"
drop datafs "/web/html/static/js/firmwareudp.deec2552a51ce8502d39.js"
drop datafs "/web/html/static/js/home.0e8e877f66462454941c.js"
drop datafs "/web/html/static/js/home.1c6985293d16903dec6e.js"
drop datafs "/web/html/static/js/home.28e6ad51d0a28041e798.js"
drop datafs "/web/html/static/js/home.4bef13d4d4db14ac6612.js"
drop datafs "/web/html/static/js/home.6a5d5124e70494696bf7.js"
drop datafs "/web/html/static/js/home.74b81a7f366acba3b1c5.js"
drop datafs "/web/html/static/js/home.952ff3011efc018aa3de.js"
drop datafs "/web/html/static/js/home.a8ae5d20cb56ec52579a.js"
drop datafs "/web/html/static/js/home.ebc7e22abe27865c8741.js"
drop datafs "/web/html/static/js/home.ec3295596571279a30e2.js"
drop datafs "/web/html/static/js/manifest.052b8644aca9ed59ad55.js"
drop datafs "/web/html/static/js/manifest.0cb91adcd52c28ae7bd5.js"
drop datafs "/web/html/static/js/manifest.189656ab2fd1f8376919.js"
drop datafs "/web/html/static/js/manifest.1e46430f201b870bfff2.js"
drop datafs "/web/html/static/js/manifest.2f8e6f5c3719537062c0.js"
drop datafs "/web/html/static/js/manifest.4159897c9551998ddb38.js"
drop datafs "/web/html/static/js/manifest.4255c2f1b3aa419ff311.js"
drop datafs "/web/html/static/js/manifest.49b62534b583473d3fad.js"
drop datafs "/web/html/static/js/manifest.697da8bf906a1a862f68.js"
drop datafs "/web/html/static/js/manifest.71150de2062932e64473.js"
drop datafs "/web/html/static/js/manifest.81ee65ff72687afa520a.js"
drop datafs "/web/html/static/js/manifest.8281b1b12e50db8f51e3.js"
drop datafs "/web/html/static/js/manifest.84a887a1ade5b52bb401.js"
drop datafs "/web/html/static/js/manifest.98eca7819375aa1e7627.js"
drop datafs "/web/html/static/js/manifest.9f98a2dcabb444fb108f.js"
drop datafs "/web/html/static/js/manifest.a324a7b56b5c703c44c7.js"
drop datafs "/web/html/static/js/manifest.ab9c4d9ad29d4753ff23.js"
drop datafs "/web/html/static/js/manifest.ae1a46d63a0e9f8b14d1.js"
drop datafs "/web/html/static/js/manifest.b674814b86e913ebb65f.js"
drop datafs "/web/html/static/js/manifest.c82da2534d8df699cf0b.js"
drop datafs "/web/html/static/js/manifest.e1a41d41c0894cb399a2.js"
drop datafs "/web/html/static/js/manifest.ea7bbea836349cd67103.js"
drop datafs "/web/html/static/js/networksetup.446ebb367a7f9cba6a47.js"
drop datafs "/web/html/static/js/networksetup.92b4b8a62b3af840e84b.js"
drop datafs "/web/html/static/js/networksetup.ba63bde5d90a48004f86.js"
drop datafs "/web/html/static/js/Newtek.25c96b7b5eae9afd159f.js"
drop datafs "/web/html/static/js/Newtek.afecf510ad86324f4a56.js"
drop datafs "/web/html/static/js/OpenSourceComponents.02064f797f1cb00d3e6b.js"
drop datafs "/web/html/static/js/softwareProtocol.17dbe7065930cedc86d9.js"
drop datafs "/web/html/static/js/softwareProtocol.e862e6729146cff5212a.js"
drop datafs "/web/html/static/js/softwareProtocol.f67e853e2b9c343f7406.js"
drop datafs "/web/html/static/js/softwareProtocol.fa8d5ee3282b38ddd07c.js"
drop datafs "/web/html/static/js/superMode.016d00d0f394ab2fdae6.js"
drop datafs "/web/html/static/js/superMode.a0bd5425e7d24f81114c.js"
drop datafs "/web/html/static/js/system-time.0183a96e538e4b9bf844.js"
drop datafs "/web/html/static/js/system-time.a6371167d26acdb50e68.js"
drop datafs "/web/html/static/js/users.0dd4f9c8e3c618c594fd.js"
drop datafs "/web/html/static/js/users.3567b36f49f9b8b0cf12.js"
drop datafs "/web/html/static/js/users.8d6a2fb4643855ca3812.js"
drop datafs "/web/html/static/js/users.9e0ca937bb2db8e298e1.js"
drop datafs "/web/html/static/js/users.e59573760d72ef65d06f.js"
drop datafs "/web/html/static/js/users.e878b56d24269a49ac6e.js"
drop datafs "/web/html/static/js/users.f999d4c67dc1e954dcf0.js"
drop datafs "/web/html/static/js/vendor.161f9f7535623ba13d16.js"
drop datafs "/web/html/static/js/vendor.1bc9cb3d92c592dd4167.js"
drop datafs "/web/html/static/js/vendor.53a4422bcd982c11aa3d.js"
drop datafs "/web/html/static/js/vendor.6be301babdfa0405e23d.js"
drop datafs "/web/html/static/js/vendor.81e453a011bbeb9a63c3.js"
drop datafs "/web/intercom/html/static/css/app.56861ad532875ac8be88a21d81666b69.css"
drop datafs "/web/intercom/html/static/js/0.57a5b4d46ec618511455.js"
drop datafs "/web/intercom/html/static/js/1.3c450ebf7cd30cfea458.js"
drop datafs "/web/intercom/html/static/js/app.93094ac82a88bebde158.js"
drop datafs "/web/intercom/html/static/js/manifest.40db10b367760e6a9b7b.js"
drop datafs "/web/intercom/html/static/js/manifest.8fcdfc8d54cddd18f389.js"
drop datafs "/web/intercom/html/static/lang/en/element-ui.json"
drop datafs "/web/intercom/html/static/lang/en/errorMsg.json"
drop datafs "/web/intercom/html/static/lang/tw/element-ui.json"
drop datafs "/web/intercom/html/static/lang/tw/errorMsg.json"
drop datafs "/web/intercom/html/static/lang/zh/element-ui.json"
drop datafs "/web/intercom/html/static/lang/zh/errorMsg.json"
drop datafs "/web/intercom/lang/en/element-ui.json"
drop datafs "/web/intercom/lang/en/errorMsg.json"
drop datafs "/web/intercom/lang/tw/element-ui.json"
drop datafs "/web/intercom/lang/tw/errorMsg.json"
drop datafs "/web/intercom/lang/zh/element-ui.json"
drop datafs "/web/intercom/lang/zh/errorMsg.json"
drop datafs "/web/kilolink/html/static/lang/en/element-ui.json"
drop datafs "/web/kilolink/html/static/lang/zh/element-ui.json"
drop fakeroot "/conv_ndi_image"
drop fakeroot "/etc/coserver/local/PtzManager"
drop fakeroot "/.stage.apt/etc/apt/apt.conf.d/apt.conf"
drop fakeroot "/.stage.apt/etc/apt/preferences.d/preferences"
drop fakeroot "/.stage.apt/etc/apt/sources.list"
drop fakeroot "/.stage.apt/usr/bin/update-alternatives"
drop fakeroot "/.stage.apt/usr/sbin/update-rc.d"
drop fakeroot "/.stage.apt/var/cache/apt/archives/lock"
drop fakeroot "/.stage.apt/var/lib/apt/extended_states"
drop fakeroot "/.stage.apt/var/lib/apt/lists/lock"
drop fakeroot "/.stage.apt/var/lib/apt/lists/_opt_Xilinx_petalinux-v2015.4-final_components_packages-repo_plnx-repo_dists_default_main_binary-cortexa9-vfp-neon_Packages"
drop fakeroot "/.stage.apt/var/log/apt/history.log"
drop fakeroot "/.stage.apt/var/log/apt/term.log"
drop fakeroot "/.targetroot.apt/etc/apt/apt.conf.d/apt.conf"
drop fakeroot "/.targetroot.apt/etc/apt/preferences.d/preferences"
drop fakeroot "/.targetroot.apt/etc/apt/sources.list"
drop fakeroot "/.targetroot.apt/usr/bin/update-alternatives"
drop fakeroot "/.targetroot.apt/usr/sbin/update-rc.d"
drop fakeroot "/.targetroot.apt/var/cache/apt/archives/lock"
drop fakeroot "/.targetroot.apt/var/lib/apt/extended_states"
drop fakeroot "/.targetroot.apt/var/lib/apt/lists/lock"
drop fakeroot "/.targetroot.apt/var/lib/apt/lists/_opt_Xilinx_petalinux-v2015.4-final_components_packages-repo_plnx-repo_dists_default_main_binary-cortexa9-vfp-neon_Packages"
drop fakeroot "/.targetroot.apt/var/log/apt/history.log"
drop fakeroot "/.targetroot.apt/var/log/apt/term.log"
drop fakeroot "/usr/lib/alsa-lib/smixer/smixer-ac97.la"
drop fakeroot "/usr/lib/alsa-lib/smixer/smixer-hda.la"
drop fakeroot "/usr/lib/alsa-lib/smixer/smixer-sbase.la"
drop fakeroot "/usr/lib/libndi.so.4"
drop fakeroot "/usr/lib/libndi.so.4.1.0"
drop fakeroot "/usr/lib/libndi.so.4.1.4"
drop fakeroot "/usr/lib/libndi.so.4.5.0"
drop fakeroot "/usr/lib/libndi.so.4.5.2"
drop fakeroot "/usr/lib/lua/5.3/sii9136drv.so"
drop fakeroot "/usr/local/bin/codec/chips/ite66021.lua"
drop fakeroot "/usr/local/bin/codec/chips/ite68051.lua"
drop fakeroot "/usr/local/bin/codec/chips/sii9136.lua"
drop fakeroot "/usr/local/bin/codec/hdmi_chips.lua"
drop fakeroot "/usr/local/bin/codec/iocfg.lua"
drop fakeroot "/usr/local/bin/led.sh"
drop fakeroot "/usr/local/bin/PtzManager/configs/ptz_configs.lua"
drop fakeroot "/usr/local/bin/PtzManager/copy_merge.lua"
drop fakeroot "/usr/local/bin/PtzManager/mod_PtzManager.lua"
drop fakeroot "/usr/local/bin/PtzManager/PtzManager.lua"
upgrade
#!/bin/sh

#Drop EULA accept status
rm -Rf /data/configs/EULA/* > /dev/null 2>&1
## Upgrading done at here.

#Drop EULA
rm -Rf /data/configs/EULA/* > /dev/null 2>&1

#Update version.
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/share/lua/$LUA_VERSION/version.lua >> $logfile 2>&1
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/local/openresty/lualib/version.lua >> $logfile 2>&1

echo "------------------------------------" >> $logfile
echo "Finished upgrading. [$WITH_ERROR errors, $WITH_WARNING warnnings.]" >> $logfile

rm -Rf $TARGET_TMP_DIR/_UPGRADE
sync
sync
exit $UPGRADE_RETURNS

